//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "clientlist.h"
#include "emule.h"
#include "otherfunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


CClientList::CClientList(){
}

CClientList::~CClientList(){
}

// xrmb : statsclientstatus
void CClientList::GetStatistics(uint32 &totalclient, int stats[], ClientsData *pData )
{
	EMULE_TRY
	if ( !pData ) 
	{
		return;
	}
	totalclient = list.GetCount();


	POSITION pos1, pos2;

	for (int i=0;i<SO_LAST;i++)
	{
		pData->m_pClients[i].RemoveAll();
		stats[i]=0;
	}
	pData->m_pClients[i].RemoveAll(); // reset the last one

	for (pos1 = list.GetHeadPosition();( pos2 = pos1 ) != NULL;){
		list.GetNext(pos1);
		CUpDownClient* cur_client =	list.GetAt(pos2);
		int version	=	0;
		
		switch (cur_client->GetClientSoft()) {
			case SO_UNKNOWN : 
				stats[SO_UNKNOWN]++;
				break;
			case SO_EDONKEY : 
				stats[SO_EDONKEY]++;
				(pData->m_pClients[SO_EDONKEY])[cur_client->GetVersion()]++;
				break;
			case SO_EDONKEYHYBRID : 
				stats[SO_EDONKEYHYBRID]++;
				(pData->m_pClients[SO_EDONKEYHYBRID])[cur_client->GetVersion()]++;
				break;
			case SO_EMULE   :
				stats[SO_EMULE]++;
				version = cur_client->GetMuleVersion();
				(pData->m_pClients[SO_EMULE])[version]++;
				break;
			case SO_OLDEMULE:
				stats[SO_OLDEMULE]++;
				version = cur_client->GetMuleVersion();
				(pData->m_pClients[SO_OLDEMULE])[version]++;
				break;
			case SO_CDONKEY : //Didn't get much time to test this "eMule Compatable" feature, so for this version they will just be added to our total.
				stats[SO_CDONKEY]++;
				version = cur_client->GetMuleVersion();
				(pData->m_pClients[SO_CDONKEY])[version]++;
				break;
			case SO_SHAREAZA:
				stats[SO_SHAREAZA]++;
				version = cur_client->GetMuleVersion();
				(pData->m_pClients[SO_SHAREAZA])[version]++;
				break;
			case SO_LMULE:
				stats[SO_LMULE]++;
				version = cur_client->GetMuleVersion();
				(pData->m_pClients[SO_LMULE])[version]++;
				break;

			case SO_MLDONKEY:
				stats[SO_MLDONKEY]++;
				(pData->m_pClients[SO_MLDONKEY])[version]++;
				break;
		}

		(pData->m_pClients[SO_LAST])[cur_client->GetDownloadState()]++;
	}

	EMULE_CATCH
}


void CClientList::AddClient(CUpDownClient* toadd,bool bSkipDupTest)
{
	EMULE_TRY

	if(toadd == NULL)
		return;

	if ( !bSkipDupTest){
		if(list.Find(toadd))
			return;
	}
	list.AddTail(toadd);

	EMULE_CATCH
}

void CClientList::RemoveClient(CUpDownClient* toremove)
{
	EMULE_TRY

	POSITION pos = list.Find(toremove);
	if (pos){
		//just to be sure...
		theApp.uploadqueue->RemoveFromUploadQueue(toremove);
		theApp.uploadqueue->RemoveFromWaitingQueue(toremove);
		theApp.downloadqueue->RemoveSource(toremove);
		list.RemoveAt(pos);
	}

	EMULE_CATCH
}

void CClientList::DeleteAll()
{
	EMULE_TRY

	theApp.uploadqueue->DeleteAll();
	theApp.downloadqueue->DeleteAll();
	POSITION pos1, pos2;
	for (pos1 = list.GetHeadPosition();( pos2 = pos1 ) != NULL;){
		list.GetNext(pos1);
		CUpDownClient* cur_client =	list.GetAt(pos2);
		list.RemoveAt(pos2);
		if(cur_client)
			delete cur_client; // recursiv: this will call RemoveClient
	}

	EMULE_CATCH
}

#ifdef OLD_SOCKETS_ENABLED
bool CClientList::AttachToAlreadyKnown(CUpDownClient** client, CClientReqSocket* sender)
{
	EMULE_TRY

	POSITION pos1, pos2;
	CUpDownClient* tocheck = (*client);
	for (pos1 = list.GetHeadPosition();( pos2 = pos1 ) != NULL;){
		list.GetNext(pos1);
		CUpDownClient* cur_client =	list.GetAt(pos2);
		// xrmb : crashfix
      	// i dont know how and why this can happen... but emule crashed here all the time for me! 
      	if (cur_client && tocheck!=cur_client && tocheck->Compare(cur_client))
		{ 
 			ASSERT ( (tocheck != cur_client) );
			if (sender){
				if (cur_client->socket){
					cur_client->socket->client = 0;
					cur_client->socket->Safe_Delete();
				}
				cur_client->socket = sender;
				tocheck->socket = 0;
			}
			*client = 0;
			delete tocheck;
			*client = cur_client;
			return true;
		}
	}

	EMULE_CATCH

	return false;
}
#endif //OLD_SOCKETS_ENABLED

CUpDownClient* CClientList::FindClientByIP(uint32 clientip,uint16 port)
{
	EMULE_TRY

	POSITION pos1, pos2;
	for (pos1 = list.GetHeadPosition();( pos2 = pos1 ) != NULL;){
		list.GetNext(pos1);
		CUpDownClient* cur_client =	list.GetAt(pos2);
		if(cur_client)
			if (cur_client->GetIP() == clientip && cur_client->GetUserPort() == port)
				return cur_client;
	}
	
	EMULE_CATCH

	return 0;
}

CUpDownClient* CClientList::FindClientByUserHash(uchar* clienthash)
{
	EMULE_TRY

	POSITION pos1, pos2;
	for (pos1 = list.GetHeadPosition();( pos2 = pos1 ) != NULL;){
		list.GetNext(pos1);
		CUpDownClient* cur_client =	list.GetAt(pos2);
		if(cur_client)
			if (!md4cmp(cur_client->GetUserHash() ,clienthash))
				return cur_client;
	}

	EMULE_CATCH

	return 0;
}

#ifdef OLD_SOCKETS_ENABLED
void CClientList::Debug_SocketDeleted(CClientReqSocket* deleted)
{
	EMULE_TRY
	POSITION pos1, pos2;
	for (pos1 = list.GetHeadPosition();( pos2 = pos1 ) != NULL;)
	{
		list.GetNext(pos1);
		CUpDownClient* cur_client =	list.GetAt(pos2);
		if (!AfxIsValidAddress(cur_client, sizeof(CUpDownClient)))
		{
			AfxDebugBreak();
		}
		if (cur_client && cur_client->socket == deleted)
		{
			AfxDebugBreak();
		}
	}

	EMULE_CATCH
}
#endif //OLD_SOCKETS_ENABLED

bool CClientList::Debug_IsValidClient(CUpDownClient* tocheck)
{
	EMULE_TRY

	if(tocheck == NULL)
		return false;

	return list.Find(tocheck);

	EMULE_CATCH

	return false;
}

// [i_a] 
void CClientList::ResetAllValueCaches() 
{
	EMULE_TRY

	if(!list.IsEmpty()) // Fix reCDVst
		for (POSITION pos = list.GetHeadPosition(); pos != 0; list.GetNext(pos)) 
		{ 
			CUpDownClient* cur_client = list.GetAt(pos); 
			ASSERT(cur_client); 
			if(cur_client)
				cur_client->ResetValueCache(); 
		}

	EMULE_CATCH
} 

//netwolf: complete sources (zegzav) 13.05.03 - BEGIN
void CClientList::GetClientListByFileID(CTypedPtrList<CPtrList, CUpDownClient*> *clientlist, const uchar *fileid)
{
	EMULE_TRY

	clientlist->RemoveAll();
	for (POSITION pos = list.GetHeadPosition(); pos != 0; ) 
	{
		CUpDownClient *cur_src= list.GetNext(pos);
		if (md4cmp(cur_src->GetUploadFileID(), fileid) == 0)
			clientlist->AddTail(cur_src);
	}
//netwolf: complete sources (zegzav) 13.05.03 - END

	EMULE_CATCH
}
