//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "types.h"
#include "partfile.h"
#include "memdc.h"
#include "titlemenu.h"
#include "Loggable.h"

#include <map>

using namespace std;

class CMuleCtrlItem
{
public:
	// Constructor
	CMuleCtrlItem(CPartFile* pFile, CUpDownClient* pSource)
		{ m_pFile = pFile; m_pSource = pSource; m_dwUpdated = 0; }
	~CMuleCtrlItem()
		{ m_Status.DeleteObject(); }

	// Methods
	bool IsFile(){ return m_pSource == NULL; }
	bool IsSource(){ return m_pSource != NULL; }
	CPartFile* GetFile(){ return m_pFile; }
	CUpDownClient* GetSource(){ return m_pSource; }
	bool IsAskedForAnotherFile();

public:
	// Public data (too lazy to hide :)
	DWORD	m_dwUpdated;
	CBitmap	m_Status;

private:
	CPartFile*		m_pFile;
	CUpDownClient*	m_pSource;
};

typedef multimap<void*, CMuleCtrlItem*> MuleMultiMap;
typedef MuleMultiMap::iterator MuleIter;
typedef pair<MuleIter, MuleIter> MuleRange;


// CDownloadListCtrl
class CDownloadListCtrl : public CMuleListCtrl, public CLoggable
{
	DECLARE_DYNAMIC(CDownloadListCtrl)

public:
	CDownloadListCtrl();
	virtual ~CDownloadListCtrl();
	virtual void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);
	void	UpdateItem(void* toupdate);
	void	Init();
	void	AddFile(CPartFile* toadd);
	void	AddSource(CPartFile* owner,CUpDownClient* source,bool notavailable);
	void	RemoveSource(CUpDownClient* source,CPartFile* owner);
	void	RemoveFile(CPartFile* toremove);
	void	ClearCompleted(const uchar* FileHash = NULL);	//SyruS clear single completed files
	virtual BOOL OnCommand(WPARAM wParam,LPARAM lParam );
	void	SetStyle();
	void	Localize();
	void	RestartWaitingDownloads(); 				// Rod
	void	RefreshList();						// Cax2
	void	SortInit(int override);					// Cax2
	bool	IsShowingSources() {return m_ShowSrc;}			// Cax2
	int8	FilesSortIndex;
	void	HideSources(CPartFile* partfile);  			// Cax2 closefile bugfix
	CString getTextList();
	bool	GetDisplayedFiles(CArray<CPartFile*,CPartFile*> *list);	//SyruS (failsafe) show completed files (0.28b)


protected:
	void	DrawFileItem(CDC *dc, int nColumn, LPRECT lpRect, CMuleCtrlItem *lpCtrlItem);
	void	DrawSourceItem(CDC *dc, int nColumn, LPRECT lpRect, CMuleCtrlItem *lpCtrlItem);
	afx_msg void OnLvnItemActivate(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg	void OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnListModified(NMHDR *pNMHDR, LRESULT *pResult);
	static int CALLBACK SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
    static int Compare(CPartFile* file1, CPartFile* file2, LPARAM lParamSort);
	static int Compare(CUpDownClient* client1,CUpDownClient* client2, LPARAM lParamSort, int sortMod);

	DECLARE_MESSAGE_MAP()
private:
	CImageList  	imagelist;
	int         	m_CurrentSortItem;
	bool        	m_SortAscending;
	bool			m_ShowSrc;  		// Cax2 
	int				m_lastSort;
	DWORD			m_sortTime;

	MuleMultiMap	m_ListData;
	CMutex			m_MutexAccess;

public:
	afx_msg void OnNMClick(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnNMDblclkDownloadlist(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	void RefreshInfo(void);
};
