//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "stdafx.h"
#include "emule.h"
#include "packets.h"
#include "UDPSocket.h"
#include "memcpy_amd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef OLD_SOCKETS_ENABLED

CUDPSocketWnd::CUDPSocketWnd(){
}

BEGIN_MESSAGE_MAP(CUDPSocketWnd, CWnd)
	ON_MESSAGE(WM_DNSLOOKUPDONE, OnDNSLookupDone)
END_MESSAGE_MAP()

LRESULT CUDPSocketWnd::OnDNSLookupDone(WPARAM wParam,LPARAM lParam)
{
	EMULE_TRY

	if(m_pOwner)
		m_pOwner->DnsLookupDone(wParam,lParam);
	return true;

	EMULE_CATCH

	return false;
};

CUDPSocket::CUDPSocket(CServerConnect* in_serverconnect)
{
	m_hWndResolveMessage = NULL;
	sendbuffer = 0;
	cur_server = 0;
	serverconnect = in_serverconnect;
	DnsTaskHandle = 0;
}

CUDPSocket::~CUDPSocket()
{
	EMULE_TRY

	if (cur_server)
		delete cur_server;
	if (sendbuffer)
		delete[] sendbuffer;
	m_udpwnd.DestroyWindow();

	EMULE_CATCH
}

bool  CUDPSocket::Create()
{
	EMULE_TRY

	VERIFY( m_udpwnd.CreateEx(0, AfxRegisterWndClass(0),_T("Emule Socket Wnd"),WS_OVERLAPPED, 0, 0, 0, 0, NULL, NULL));
	m_hWndResolveMessage = m_udpwnd.m_hWnd;
	m_udpwnd.m_pOwner = this;
	return CAsyncSocket::Create(0,SOCK_DGRAM,FD_READ);

	EMULE_CATCH
	return false;
}

void CUDPSocket::OnReceive(int nErrorCode)
{
	EMULE_TRY

	char buffer[5000];
	CString serverbuffer;
	uint32 port;
	int32 length = ReceiveFrom(buffer,5000,serverbuffer,port);
	if ((uint8)buffer[0] == OP_EDONKEYPROT && length != SOCKET_ERROR)
		ProcessPacket(buffer+2,length-2,buffer[1],serverbuffer.GetBuffer(),port);
	else if (((uint8)buffer[0] == OP_EMULEPROT) && length != SOCKET_ERROR)
		ProcessExtPacket(buffer+2,length-2,buffer[1],serverbuffer.GetBuffer(),port);
	
	EMULE_CATCH
}

bool CUDPSocket::ProcessPacket(char* packet, int16 size, int8 opcode, TCHAR* host, uint16 port)
{
	EMULE_TRY

	try{
		CServer* update;
		update = theApp.serverlist->GetServerByAddress( host, port-4 );
		if( update ){
			update->ResetFailedCount();
			theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*update );
		}
		switch(opcode){
			case OP_GLOBSEARCHRES:{
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				theApp.emuledlg->searchwnd.AddUDPResult(theApp.searchlist->ProcessUDPSearchanswer(packet,size));
				break;
			}
			case OP_GLOBFOUNDSOURCES:{
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				CSafeMemFile sources((BYTE*)packet,size);
				uchar fileid[16];
				sources.Read(fileid,16);
				if (CPartFile* file = theApp.downloadqueue->GetFileByID(fileid))
					file->AddSources(sources,inet_addr(host),port);
				else
					theApp.serverlist->CancelUDPBroadcast();				
				break;
			}
 			case OP_GLOBSERVSTATRES:{
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				if( size < 12 || update == NULL )
					return true;
				uint32 challenge;
				uint32 cur_user;
				uint32 cur_files;
				#ifndef AMD
				memcpy(&challenge,packet,4);
				if ( challenge != update->GetChallenge() ) return true; 
				memcpy(&cur_user,packet+4,4);
				memcpy(&cur_files,packet+8,4);
				#else
				memcpy_amd(&challenge,packet,4);
				if ( challenge != update->GetChallenge() ) return true; 
				memcpy_amd(&cur_user,packet+4,4);
				memcpy_amd(&cur_files,packet+8,4);
				#endif
				uint32 cur_maxusers = 0;
				if (size >= 16) {
					#ifndef AMD
					memcpy(&cur_maxusers, packet+12,4);
					#else
					memcpy_amd(&cur_maxusers, packet+12,4);
					#endif
				}
				if( update ){
					update->SetPing( ::GetTickCount() - update->GetLastPinged() );
					update->SetUserCount( cur_user );
					update->SetFileCount( cur_files );
					update->SetMaxUsers( cur_maxusers );
					theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*update );
				}
				break;
			}

 			case OP_SERVER_DESC_RES:{
				if(!update)
					return true;
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				CSafeMemFile srvinfo((BYTE*)packet,size);
				uint16 stringlen;
				srvinfo.Read(&stringlen,2);
				TCHAR* name = new TCHAR[stringlen+1];
				srvinfo.Read(name,stringlen);
				name[stringlen] = 0;
				srvinfo.Read(&stringlen,2);
				TCHAR* desc = new TCHAR[stringlen+1];
				srvinfo.Read(desc,stringlen);
				desc[stringlen] = 0;
				
				update->SetDescription(desc);
				update->SetListName(name);

				delete name;
				delete desc;				
				break;
			}
			default:
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				return false;
		}
		return true;
	}
	catch(CFileException* error){
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(false,IDS_ERR_UDP_MISCONF);
		error->Delete();
		return false;
	}
	catch(...){
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(false,IDS_ERR_UDP_MISCONF);
		return false;
	}

	EMULE_CATCH

	return false;
}

bool CUDPSocket::ProcessExtPacket(char* packet, int16 size, int8 opcode, TCHAR* host, uint16 port)
{
	EMULE_TRY

	try{
	switch(opcode){
		/*
		case OP_UDPVERIFYUPREQ:{
			ASSERT (size == 6);
			if (size == 6){
				uint32 checkclientip = 0;
				uint16 checkclientport = 0;
				memcpy(&checkclientip,packet,4);
				memcpy(&checkclientport,packet,2);
				if (theApp.clientlist->VerifyUpload(checkclientip,checkclientport)){
					Packet answer(OP_UDPVERIFYUPA,0,OP_EMULEPROT);
					SendTo(answer.GetUDPHeader(),2,port,host);
				}
			}
			break;
		}*/
		case OP_UDPVERIFYUPA:{
			theApp.downloadqueue->AddDownDataOverheadOther(size);
			break;
		}
		default:
			theApp.downloadqueue->AddDownDataOverheadOther(size);
			return false;
	}
	return true;
	}
	catch(...){
		OUTPUT_DEBUG_TRACE();
		theApp.emuledlg->AddDebugLogLine(false,IDS_ERR_UDPEXT);
		return false;
	}

	EMULE_CATCH

	return false;
}

void CUDPSocket::AsyncResolveDNS(LPCTSTR lpszHostAddress, UINT nHostPort)
{
	EMULE_TRY

	m_lpszHostAddress = lpszHostAddress;
	m_nHostPort = nHostPort;
	if (DnsTaskHandle)
		WSACancelAsyncRequest(DnsTaskHandle);
	DnsTaskHandle = NULL;
	// see if we have a ip already
	USES_CONVERSION;
	SOCKADDR_IN sockAddr;
	memset(&sockAddr,0,sizeof(sockAddr));
	LPSTR lpszAscii = T2A((LPTSTR)m_lpszHostAddress);
	sockAddr.sin_family = AF_INET;
	sockAddr.sin_addr.s_addr = inet_addr(lpszAscii);
	sockAddr.sin_port = htons((u_short)m_nHostPort);

	// backup for send socket
	m_SaveAddr = sockAddr;

	if (sockAddr.sin_addr.s_addr == INADDR_NONE){
		/* Resolve hostname "hostname" asynchronously */ 
		memset(DnsHostBuffer, 0, sizeof(DnsHostBuffer));

		USES_CONVERSION;
		DnsTaskHandle = WSAAsyncGetHostByName(
			m_hWndResolveMessage,
			WM_DNSLOOKUPDONE,
			CT2A(lpszHostAddress),
			DnsHostBuffer,
			MAXGETHOSTSTRUCT);

		if (DnsTaskHandle == 0){
			delete[] sendbuffer;
			sendbuffer = 0;
			delete cur_server;
			cur_server = 0;
#ifdef _DEBUG
			AfxMessageBox(_T("LOOKUPERROR DNSTASKHANDLE = 0"));
#endif
		}
	}
	else{
		SendBuffer();
	}

	EMULE_CATCH
}

void CUDPSocket::DnsLookupDone(WPARAM wp, LPARAM lp)
{
	EMULE_TRY

	DnsTaskHandle = NULL;

	/* An asynchronous database routine completed. */
	if (WSAGETASYNCERROR(lp) != 0){
		if (sendbuffer)
			delete[] sendbuffer;
		sendbuffer = 0;
		if (cur_server)
			delete cur_server;
		cur_server = 0;
		return;
	}
	if (m_SaveAddr.sin_addr.s_addr == INADDR_NONE){
		// get the structure length
		int iBufLen = WSAGETASYNCBUFLEN(lp);
		LPHOSTENT lphost = (LPHOSTENT)malloc(iBufLen);
		#ifndef AMD
		memcpy(lphost, DnsHostBuffer, iBufLen);
		#else
		memcpy_amd(lphost, DnsHostBuffer, iBufLen);
		#endif
		m_SaveAddr.sin_addr.s_addr = ((LPIN_ADDR)lphost->h_addr)->s_addr;
		free(lphost);
		// also reset the receive buffer
		memset(DnsHostBuffer, 0, sizeof(DnsHostBuffer));
	}
	if (cur_server){
		CServer* update = theApp.serverlist->GetServerByAddress(cur_server->GetAddress(),cur_server->GetPort());
		if (update)
			update->SetID(m_SaveAddr.sin_addr.S_un.S_addr);
		SendBuffer();
	}
	EMULE_CATCH
}

void CUDPSocket::SendBuffer()
{
	EMULE_TRY

	if(cur_server && sendbuffer){
		SendTo(sendbuffer,sendblen,(SOCKADDR*)&m_SaveAddr, sizeof(m_SaveAddr));
		delete[] sendbuffer;
		sendbuffer = 0;
		delete cur_server;
		cur_server = 0;
	}

	EMULE_CATCH
}

void CUDPSocket::SendPacket(Packet* packet,CServer* host)
{
	EMULE_TRY

	cur_server = new CServer(*host);
	sendbuffer = new char[packet->size+2];
	#ifndef AMD
	memcpy(sendbuffer,packet->GetUDPHeader(),2);
	memcpy(sendbuffer+2,packet->pBuffer,packet->size);
	#else
	memcpy_amd(sendbuffer,packet->GetUDPHeader(),2);
	memcpy_amd(sendbuffer+2,packet->pBuffer,packet->size);
	#endif
	sendblen = packet->size+2;
	AsyncResolveDNS(cur_server->GetAddress(),cur_server->GetPort()+4);

	EMULE_CATCH
}

#endif //OLD_SOCKETS_ENABLED
